@@ -5,8 +5,9 @@
 # complains several times about module random having no attribute
 # randrange, and then Python hangs.
 
-import thread
-from test.test_support import verbose, TestSkipped, TestFailed
+import unittest
+from test.test_support import verbose, TestFailed, import_module
+thread = import_module('thread')
 
 critical_section = thread.allocate_lock()
 done = thread.allocate_lock()
@@ -30,11 +31,10 @@ def test_import_hangers():
     if verbose:
         print "testing import hangers ...",
 
-    from test import threaded_import_hangers
-
+    import test.threaded_import_hangers
     try:
-        if threaded_import_hangers.errors:
-            raise TestFailed(threaded_import_hangers.errors)
+        if test.threaded_import_hangers.errors:
+            raise TestFailed(test.threaded_import_hangers.errors)
         elif verbose:
             print "OK."
     finally:
@@ -57,7 +57,7 @@ def test_main():        # magic name!  see above
     import imp
     if imp.lock_held():
         # This triggers on, e.g., from test import autotest.
-        raise TestSkipped("can't run when import lock is held")
+        raise unittest.SkipTest("can't run when import lock is held")
 
     done.acquire()
     for N in (20, 50) * 3:
